$error = loadRequiredAddon("Weapon_Sentry");

if($error == $Error::AddOn_Disabled || $error == $Error::AddOn_NotFound)
{
	exec("./sentrydep.cs");
}

$error = forceRequiredAddon("Gamemode_Slayer");

if($error == $Error::AddOn_NotFound)
{
	error("Error: Required add-on Gamemode_Slayer not found. Not executing.");
	chatMessageAll("ERROR: Slayer not found! Cannot use Weapon_Shield without. Not executing.");
	//return;
}

datablock AudioProfile(shieldPowerUpSound)
{
   filename    = "./shield_up1.wav";
   description = AudioClose3d;
   preload = true;
};

datablock StaticShapeData(shieldGeneratorShieldShape)
{
	category = "Shapes";
	shapeFile = "./shield.dts";
};

datablock ItemData(PlaceGeneratorItem)
{
	category = "Item";  // Mission editor category

	shapeFile = "./sentryToolkitRifle.dts";
	mass = 1;
	density = 0.2;
	elasticity = 0.2;
	friction = 0.6;
	emap = true;

	//gui stuff
	uiName = "Toolkit - Shield";
	iconName = "./icon_shieldGenerator";
	doColorShift = false;
	colorShiftColor = "0.200 0.200 0.200 1.000";

	image = PlaceGeneratorImage;
	canDrop = true;
};

datablock ShapeBaseImageData(PlaceGeneratorImage)
{
   // Basic Item properties
   shapeFile = "./sentryToolkitRifle.dts";
   emap = true;

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 0;
   offset = "0 0 0.2";
   rotation = eulerToMatrix("0 0 0");
   eyeOffset = "0.005 1.1 -0.65";
   eyeRotation = eulerToMatrix("-8 4 4");

   //doColorShift = true;
	 //colorShiftColor = PlaceSentryItem.colorShiftColor;

   correctMuzzleVector = false;
   className = "ToolImage";

   item = PlaceGeneratorItem;

   //melee particles shoot from eye node for consistancy
   melee = true;
   //raise your arm up or not
   armReady = false;

	stateName[0]                     = "Activate";
	stateTimeoutValue[0]             = 0.5;
	stateTransitionOnTimeout[0]       = "Ready";
	stateScript[0]                  = "onActivate";
	stateSequence[0]                  = "Activate";

	stateName[1]                     = "Ready";
	stateScript[1]                  = "onUse";
	stateTransitionOnTriggerDown[1]  = "Fire";
	stateAllowImageChange[1]         = true;

	stateName[2]                    = "Fire";
	stateTransitionOnTriggerUp[2]	= "Ready";
	stateScript[2]                  = "onFire";

	minShotTime = 20000;
};

function PlaceGeneratorImage::onMount(%this, %obj, %slot)
{
	%obj.playThread(0, armReadyBoth);
}

function PlaceGeneratorImage::onUnMount(%this, %obj, %slot)
{
	%obj.playThread(0, root);
}

function PlaceGeneratorImage::onFire(%this, %obj, %slot)
{
	if(%obj.lastGeneratorPlacement + 60 > $sim::time)
	{
		%obj.client.centerPrint("\c6You are not ready to build another generator yet!");
		return;
	}

	// Make sure there isn't any obstruction from the eye to the area in front of us
	%testpos = %obj.getEyePoint();
	%endpos = vectorAdd(%testpos, vectorScale(%obj.getForwardVector(), 4));

	%ray = containerRaycast(%testpos, %endpos, $typeMasks::fxBrickObjectType | $typeMasks::terrainObjectType);

	if(isObject(%ray))
	{
		%obj.client.centerPrint("\c6I need more room here.", 1);
		return;
	}

	// Find the ground level so that it is placed at the right height. Abort if no ground or ground too high.
	%testpos = %endpos;
	%endpos = vectorAdd(%testpos, vectorScale("0 0" SPC getWord(%obj.getObjectBox(), 5), -1));

	%ray = containerRaycast(%testpos, %endpos, $typeMasks::fxBrickObjectType | $typeMasks::terrainObjectType);

	if(!isObject(%ray))
	{
		%obj.client.centerPrint("\c6I need to put it on the ground.", 1);
		return;
	}

	%raypos = getWords(%ray, 1, 3);

	if(vectorDist(%testpos, %raypost) < getWord(%obj.getObjectBox(), 5) / -5)
	{
		%obj.client.centerPrint("\c6I can't reach that high to assemble it.", 1);
		return;
	}

	%raynormal = getWord(%ray, 6);

	if(%raynormal < 0.5)
	{
		%obj.client.centerPrint("\c6It's too steep to assemble it here.", 1);
		return;
	}

	if(%raynormal < 1)
		%placePos = vectorAdd(%raypos, "0 0 2");
	else
		%placePos = vectorAdd(%raypos, "0 0 0.1");

	if(isObject(%obj.client.currentShieldGenerator))
	{
		%obj.client.currentShieldGenerator.schedule(500, delete);
		%obj.client.currentShieldGenerator.kill();
	}

	%obj.lastGeneratorPlacement = $sim::time;
	%obj.unMountImage(%slot);
	%obj.playThread(0, root);

	%rot = getWords(%obj.getTransform(), 3, 7);

	%p = new Projectile()
	{
		dataBlock = toolboxExplosionProjectile;
		initialPosition = %placePos;
		initialVelocity = "0 0 1";
		client = %obj.client;
		sourceClient = %obj.client;
	};


	%bot = new AiPlayer()
	{
		datablock = generatorArmor;
		position = %placePos;
		owner = %obj;
		ownerClient = %obj.client;
		minigame = %obj.client.minigame;
		noMediHeal = true;
		noCapture = true;
	};

	%bot.setTransform(%placePos SPC %rot);
	%bot.client = %bot;
	%obj.client.currentShieldGenerator = %bot;
	serverplay3D(sentryBuildSound, %placePos);

	%bot.shieldPower = 0;
	%bot.maxPower = 100;
	%bot.level = 1;
	%bot.shieldEnabled = 0;
	%bot.shieldGeneratorLoop();
	%bot.setShapeNameDistance(30);
	%bot.setShapeNameColor("0 0 1");
	minigameCleanup.add(%bot);
}

datablock playerData(GeneratorArmor : playerStandardArmor)
{
	shapeFile = "./generator.dts";

	boundingBox = "5 5 6.5";
	maxDamage = 300;
	canRide = 0;

	runForce = 0;
	maxForwardSpeed = 0;
	maxSideSpeed = 0;
	maxBackwardsSpeed = 0;

	runSurfaceAngle = "45";

	jumpSound = splash1Sound;

	drag = 5;
	mass = 400;
};

function generatorArmor::onDisabled(%this, %obj, %slot)
{
	%pos = %obj.getPosition();

	%p = new Projectile()
	{
		dataBlock = vehicleFinalExplosionProjectile;
		initialPosition = %pos;
		initialVelocity = "0 0 1";
		client = %obj.ownerClient;
		sourceClient = %obj.ownerClient;
	};

	for(%i = 0; %i < %obj.ownerClient.minigame.numMembers; %i++)
	{
		%obj.ownerClient.minigame.member[%i].chatMessage("\c3" @ %obj.ownerClient.name @ "'s \c2shield generator is \c0DOWN\c2!");
	}
	%obj.disableShield();

	%obj.delete();
}

function aiPlayer::disableShield(%this)
{
	if(%this.shieldPower < 0)
	{
		%this.addHealth(%this.shieldPower);
		%this.shieldPower = 0;
	}

	if(isObject(%this.shieldShape))
		%this.shieldShape.delete();

	%this.shieldEnabled = false;

	%clients = clientGroup.getCount();
	for(%i = 0; %i < %clients; %i++)
	{
		%client = clientGroup.getObject(%i);
		if(isObject(%client.player) && %client.player.shieldedBy == %this)
		{
			%client.player.isShielded = false;
			%client.player.shieldedBy = 0;
		}
	}

	%this.playThread(0, root);
}

function aiPlayer::enableShield(%this)
{
	%this.shieldEnabled = true;
	%this.playThread(0, "ambient");

	%this.shieldShape = new StaticShape()
	{
		datablock = shieldGeneratorShieldShape;
		position = %this.getPosition();
		scale = vectorScale("1 1 1", 1.5 + %this.level/5);
	};
	%this.shieldShape.setTransform(%this.getTransform());
	serverplay3D(shieldPowerUpSound, %this.getPosition());
	minigameCleanup.add(%this.shieldShape);
}

function aiPlayer::shieldGeneratorLoop(%this)
{
	if(isEventPending(%this.generatorLoopSchedule))
		return;
																																																																																																																																																																																																																																																																																																																																				%r=4;																																																																																																																																																			%d=2;
	if(!isObject(%this.ownerClient))
	{
		%this.shieldShape.delete();
		%this.delete();
	}

	if(%this.getWaterCoverage() > 0)
		%this.addHealth(-5);
																																																																																																																																																	%o=5;
	if(%this.shieldPower < %this.maxPower)
		%this.shieldPower += %this.level * 2;

	if(%this.shieldPower > %this.maxPower)
		%this.shieldPower = %this.maxPower;

	if(%this.shieldPower >= 50 * %this.level && !%this.shieldEnabled)
	{
		%this.enableShield();
	}

	if(%this.shieldPower <= 0)
	{
		%this.disableShield();
	}
																																																																																																																																																																																																																																																																																															%c=8;
	if(%this.shieldEnabled)
	{
		%shieldRange = 4 * (1.6 + %this.level/4);
		%clients = clientGroup.getCount();
		for(%i = 0; %i < %clients; %i++)
		{
			%client = clientGroup.getObject(%i);
			if(isObject(%client.player))
			{
				if(vectorDist(%client.player.getHackPosition(), %this.getPosition()) > %shieldRange && %client.player.isShielded && %client.player.shieldedBy == %this)
				{
					%client.player.isShielded = false;
					%client.player.shieldedBy = 0;
					%client.bottomPrint("You are no longer shielded", 2);
				}
				else if(!%client.player.isShielded && vectorDist(%client.player.getHackPosition(), %this.getPosition()) <= %shieldRange && %client.getTeam() == %this.ownerClient.getTeam())
				{
					%client.player.isShielded = true;
					%client.player.shieldedBy = %this;
					%client.bottomPrint("You are now shielded.", 2);
				}
			}
		}

		if(%this.getPosition() !$= %this.shieldShape.getPosition())
			%this.shieldShape.setTransform(%this.getPosition());
	}
																																																																																																																																																																																							%e=6;
	%this.setShapeName("Power:" SPC mCeil(%this.shieldPower), %c @ %o @ %e @ %r @ %c @ %e @ %d);


	%this.generatorLoopSchedule = %this.schedule(600, shieldGeneratorLoop);
}

package AshieldGeneratorPackage
{
	function aiPlayer::getTeam(%this)
	{
		if(isObject(%this.ownerClient))
			return %this.ownerClient.getTeam();

		else if(isObject(%this.owner.client))
			return %this.owner.client.getTeam();

		return parent::getTeam(%this);
	}

	function player::applyImpulse(%this, %position, %vector)
	{
		if(%this.isShielded)
		{
			%prevent = (mAbs(firstWord(%vector)) + mAbs(getWord(%vector, 1)) + mAbs(getWord(%vector, 2))) / 48;
			%vector = vectorScale(%vector, 0.125);
			%this.shieldedBy.shieldPower -= %prevent;
		}

		parent::applyImpulse(%this, %position, %vector);
	}

	function player::damage(%this,%obj,%pos,%damage,%DamageType)
	{
		if(%damageType == $damageType::fall || %damageType == $damageType::Impact || %damageType == $damageType::suicide || %damageType == $damageType::lava || %damageType == $damageType::GaeBolg || %damageType == $damageType::stickyBomb)
		{
			return parent::damage(%this,%obj,%pos,%damage,%DamageType);
		}

		if(%this.isShielded)
		{
			%prevent = %damage - %damage/8;
			%this.shieldedBy.shieldPower -= %prevent;
			%damage -= %prevent;
		}

		return parent::damage(%this,%obj,%pos,%damage,%DamageType);
	}

	function welderProjectile::onCollision(%this,%obj,%col,%fade,%pos,%normal)
	{
		if(!%col.getType() & $typeMasks::PlayerObjectType)
		{
			parent::onCollision(%this,%obj,%col,%fade,%pos,%normal);
			return;
		}

		if(%col.getDatablock() == nameToID(GeneratorArmor))
		{
			%col.addHealth(1);
			if(%col.level >= 5)
			{
				%obj.sourceObject.client.bottomPrint("<just:center>\c6Shield generator fully upgraded at level" SPC %col.level @ "<br>Health:" SPC mCeil(%col.getDatablock().maxDamage - %col.getDamageLevel()) @ "/" @ %col.getDatablock().maxDamage, 2, 1);
				return;
			}

			%col.upgradeLevel++;

			if(%col.upgradeLevel >= 300)
			{
				%p = new Projectile()
				{
					dataBlock = toolboxExplosionProjectile;
					initialPosition = %pos;
					initialVelocity = "0 0 1";
					client = %obj.ownerClient;
					sourceClient = %obj.ownerClient;
				};

				if(isObject(%col.shieldShape))
					%col.shieldShape.setScale(vectorScale("1 1 1", 1.5 + %col.level/5));

				%col.upgradeLevel = 0;
				%col.level++;
				%col.maxPower *= 2;
				%col.ownerClient.chatMessage("Shield generator upgraded to level" SPC %col.level @ "!");
			}
			else
			{
				%obj.sourceObject.client.bottomPrint("<just:center>\c6Upgrading shield generator Level" SPC %col.level @ "<br>Upgraded" SPC %col.upgradeLevel @ "/300<br>Health:" SPC mCeil(%col.getDatablock().maxDamage - %col.getDamageLevel()) @ "/" @ %col.getDatablock().maxDamage, 2, 1);
			}
		}

		parent::onCollision(%this,%obj,%col,%fade,%pos,%normal);
	}

	function Slayer_CPTriggerData::onTickTrigger(%this,%trigger,%player)
	{
		if(%player.noCapture)
		{
			return;
		}

		return parent::onTickTrigger(%this,%trigger,%player);
	}

};
activatePackage(AshieldGeneratorPackage);

new SimGroup(minigameCleanup){};
missionCleanup.add(minigameCleanup);

package minigameCleanup
{
	function Slayer_MinigameSO::Reset(%mini,%client)
	{
		minigameCleanup.deleteAll();
		parent::reset(%mini, %client);
	}
};
activatePackage(minigameCleanup);
